package threadExamples;

import java.util.Random;

/**
 * Tri d'un tableau d'entiers Version mono-thread
 */
public class Trieur implements Runnable {

	private int[] t;

	int thread_debut;
	int thread_fin;

	private Trieur(int[] t) {
		this.t = t;
	}

	private Trieur(int[] t, int debut, int fin) {
		this.t = t;
		thread_debut  = debut;
		thread_fin = fin;
	}

	/**
	 * Trie un tableau d'entiers par ordre croissant
	 * 
	 * @param t
	 *            tableau  trier
	 */
	public static void trier(int[] t) {
		Trieur tableau = new Trieur(t);
		tableau.trier(0, t.length - 1);
	}

	public void run() {
		if (thread_fin - thread_debut < 4000) {
			Trieur tableau = new Trieur(t);
			tableau.trier(thread_debut, thread_fin);
		} else {
			int thread_milieu = thread_debut + (thread_fin - thread_debut) / 2;
			Thread t1 = new Thread(new Trieur(t,thread_debut, thread_milieu));
			Thread t2 = new Thread(new Trieur(t,thread_milieu + 1, thread_fin));
			t1.start();
			t2.start();
			try {
				t1.join();
				t2.join();
			} catch (InterruptedException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			triFusion(thread_debut, thread_fin);
		}
	}

	/**
	 * Trie une tranche de t
	 * 
	 * @param debut
	 *            indice du dbut de la partie  trier
	 * @param debut
	 *            indice de la fin de la partie  trier
	 */
	private void trier(int debut, int fin) {
		if (fin - debut < 2) {
			if (t[debut] > t[fin]) {
				echanger(debut, fin);
			}
		} else {
			int milieu = debut + (fin - debut) / 2;
			trier(debut, milieu);
			trier(milieu + 1, fin);
			triFusion(debut, fin);
		}
	}

	/**
	 * Echanger t[i] et t[j]
	 */
	private void echanger(int i, int j) {
		int valeur = t[i];
		t[i] = t[j];
		t[j] = valeur;
	}

	/**
	 * Fusionne 2 tranches dj tries du tableau t. - 1re tranche : de debut 
	 * milieu - 2me tranche : de milieu + 1  fin
	 * 
	 * @param milieu
	 *            indique le dernier indice de la 1re tranche
	 */
	private void triFusion(int debut, int fin) {
		// tableau o va aller la fusion
		int[] tFusion = new int[fin - debut + 1];
		int milieu = (debut + fin) / 2;
		// Indices des lments  comparer
		int i1 = debut, i2 = milieu + 1;
		// indice de la prochaine case du tableau tFusion  remplir
		int iFusion = 0;
		while (i1 <= milieu && i2 <= fin) {
			if (t[i1] < t[i2]) {
				tFusion[iFusion++] = t[i1++];
			} else {
				tFusion[iFusion++] = t[i2++];
			}
		}
		if (i1 > milieu) {
			// la 1re tranche est puise
			for (int i = i2; i <= fin;) {
				tFusion[iFusion++] = t[i++];
			}
		} else {
			// la 2me tranche est puise
			for (int i = i1; i <= milieu;) {
				tFusion[iFusion++] = t[i++];
			}
		}
		// Copie tFusion dans t
		for (int i = 0, j = debut; i <= fin - debut;) {
			t[j++] = tFusion[i++];
		}
	}

	public static void main(String[] args) {
		int N = 2000000;
		int[] t1 = new int[N];
		int[] t2 = new int[N];
		// fill A and B with random values
		Random rand = new Random();
		int min = -N, max = N;

		// nextInt is normally exclusive of the top value,
		// so add 1 to make it inclusive
		for (int i = 0; i < N; i++) {
				int x = rand.nextInt(max - min + 1) + min;
				t1[i] = x;
				t2[i] = x;
		}
		
		// -------------------------------------------
		// Single thread
		// -------------------------------------------
		long startTime = System.currentTimeMillis();

		Trieur.trier(t1);

		long stopTime = System.currentTimeMillis();
		long elapsedTime = stopTime - startTime;
		System.out.println("Time with single thread: " + elapsedTime);

		// -------------------------------------------
		// Multi-thread
		// -------------------------------------------
		long startTimeThreads = System.currentTimeMillis();

		Thread Thread2 = new Thread(new Trieur(t2,0, N-1));
		Thread2.start();
		try {
			Thread2.join();
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		long stopTimeThreads = System.currentTimeMillis();
		long elapsedTimeThreads = stopTimeThreads - startTimeThreads;
		System.out.println("Time with threads: "
				+ elapsedTimeThreads);

		System.out.println();
	}
}
