package threadExamples;

import java.util.Random;

/*
 *-----------------------------------------------------------
 * Program    : Matrix Multiplication Using Threading in JAVA
 * Written by : Hafiz Muhammad Ahmar
 * Date          : 22/11/2011
 * Modified by : Goran Frehse, 03/2014
 * Url            :  www.allfree-codes.blogspot.com
 * Description: Create A java Threading program for multiplying Matrix.
 *-----------------------------------------------------------
 */
public class matrixmult {
 
	public static void main(String args[]) {
		int row;
		int col;
		final int N = 1000;
		int A[][] = new int[N][N];
		int B[][] = new int[N][N];
		int C[][] = new int[N][N];
		int threadcount = 0;
		Thread[] thrd = new Thread[N];

		// fill A and B with random values
		Random rand = new Random();
		int min = -10, max = 10;

		// nextInt is normally exclusive of the top value,
		// so add 1 to make it inclusive
		for (row = 0; row < N; row++) {
			for (col = 0; col < N; col++) {
				A[row][col] = rand.nextInt(max - min + 1) + min;
				B[row][col] = rand.nextInt(max - min + 1) + min;
			}
		}

		// -------------------------------------------
		// Single thread
		// -------------------------------------------
		long startTime = System.currentTimeMillis();
		for (row = 0; row < N; row++) {
			// Note: The order of the following loops has
			// a drastic effect on performance
			for (int k = 0; k < B.length; k++) {
				for (col = 0; col < N; col++) {
					C[row][col] += A[row][k] * B[k][col];
				}
			}
		}
		long stopTime = System.currentTimeMillis();
		long elapsedTime = stopTime - startTime;
		System.out.println("Time with single thread: " + elapsedTime);

		// -------------------------------------------
		// Multi-thread
		// -------------------------------------------
		long startTimeThreads = System.currentTimeMillis();
		try {
			for (row = 0; row < N; row++) {
				// creating thread for multiplications
				col = 0;
				thrd[threadcount] = new Thread(new WorkerTh(row, col, A, B, C));
				thrd[threadcount].start(); // thread start

				threadcount++;

			}

			for (int i = 0; i < threadcount; i++) {
				thrd[i].join(); // joining threads
			}
		} catch (InterruptedException ie) {
		}

		long stopTimeThreads = System.currentTimeMillis();
		long elapsedTimeThreads = stopTimeThreads - startTimeThreads;
		System.out.println("Time with " + threadcount + " threads: "
				+ elapsedTimeThreads);

		// printing resulting matrix C after multiplication
		boolean output = false;
		if (output) {
			System.out.println(" Resulting C Matrix : ");
			for (row = 0; row < N; row++) {
				for (col = 0; col < N; col++) {
					System.out.print("  " + C[row][col]);
				}
				System.out.println();
			}
		}
	}

}

class WorkerTh implements Runnable {
	private int row;
	private int col;
	private int A[][];
	private int B[][];
	private int C[][];

	public WorkerTh(int row, int col, int A[][], int B[][], int C[][]) {
		this.row = row;
		this.col = col;
		this.A = A;
		this.B = B;
		this.C = C;
	}

	@Override
	public void run() {
		for (int k = 0; k < B.length; k++) {
			for (col = 0; col < B[0].length; col++) {
				C[row][col] += A[row][k] * B[k][col];
			}
		}
	}

}
